==> ouc_api-1.1.tar.gz: ouc_api-1.1/ouc_api/jwCrawl.py <==
from .baseApi import BaseAPI
import random
from .urls import URL
import base64
from .utils.jkingo import md5_hash_passwd, encode_param
from .referers import Rerfer
import re
from bs4 import BeautifulSoup
from .uia import UIA
import redis
import lxml
import json


def cal_gpa(s):
    try:
        s = float(s)
        if 95 <= s <= 100:  # 95 - 100
            result = 4.0
        elif 90 <= s < 95:  # 90 - 94
            result = 3.9
        elif 85 <= s < 90:  # 85 - 89
            result = 3.7
        elif 81 <= s < 85:  # 810 - 84
            result = 3.3
        elif 78 <= s < 81:  # 78 - 79
            result = 3.0
        elif 75 <= s < 78:  # 75 -77
            result = 2.7
        elif 71 <= s < 75:  # 71- 74
            result = 2.3
        elif 68 <= s < 71:  # 68-70
            result = 2.0
        elif 64 <= s < 68:  # 64 - 67
            result = 1.7
        elif 60 <= s < 64:  # 60 - 63
            result = 1.0
        else:
            result = 0.0
    except:
        if s == "优秀":
            result = 3.9
        elif s == "良好":
            result = 3.3
        elif s == "中等":
            result = 2.3
        elif s == "合格":
            result = 1.0
        else:
            result = 0.0
    return result


class JwCrawl(UIA):
    def __init__(self, xh=None, password=None):
        super(JwCrawl, self).__init__()
        self.urls = [
            "http://222.195.158.207/",
        ]
        self.host = "http://222.195.158.207/"
        self.sessionId = ""
        self.set_timeout(30)
        if xh and password:
            self.xh = xh
            self.login(xh, password)

    def test_host(self):
        num = 0
        while True:
            try:
                slen = len(self.urls)  # 每次都测一下长度，每次长度都会变化。
                if slen == 0:
                    self.host = ""
                    print("无可用接口")
                    break
                num = random.randrange(0, slen)  # 随机选择一个网站
                URL = self.urls[num] + "cas/logon.action"
                self.request(URL)
                if self.get_code() == 200:
                    self.host = self.urls[num]
                    self.sessionId = self.get_cookie("JSESSIONID")  # sessionid
                    break
                else:
                    del self.urls[num]  # 该网站不可用
            except:
                del self.urls[num]  # 该网站不可用

    def get_token(self):  # 使用正则比字符串处理慢了15倍，因此采用字符串处理的方式进行。
        url = URL.get_jw_token_url(self.host)
        content = self.request(url)
        content = content.split("\n")
        _deskey = content[2].replace("var _deskey = '", "").replace("';\r", "")
        _nowtime = content[3].replace("var _nowtime = '", "").replace("';\r", "")
        # end = time.time()
        # print(end - now)
        # print(_deskey)
        # print(_nowtime)
        # d = re.findall("= (.+);", content)
        # _deskey = d[0].replace('\'', '')
        # _nowtime = d[1].replace('\'', '')
        # end = time.time()
        # print(end-now)
        data = {'_deskey': _deskey, '_nowtime': _nowtime}
        return data

    def login_cache(self, username, cookie=None):
        r = redis.Redis()
        cache = r.get(username + "cookie")
        if cookie is None and cache:
            cookie = cache.decode()
            if cookie:
                self.set_cookie("JSESSIONID", cookie)
                content = self.request(self.host + "frame/desk/showSystemInfo.action")
                if username in content:
                    return True
                else:
                    return False
        elif username and cookie:
            r.set(username + "cookie", cookie, 1800)

    def login(self, username="", password="", verify=""):
        # self.test_host() # 如果不需要token的话，目前这部是可用省略的，
        # if self.login_cache(username):
        #     return dict(status="200")
        # if self.sessionId is None:
        #     self.sessionId = ""
        # if not username and password:
        #     return None
        # u = base64.b64encode((username + ";;" + self.sessionId).encode(encoding="utf8")).decode(encoding="utf8")
        # post_data = {
        #     '_u': base64.b64encode((username + ";;" + self.sessionId).encode(encoding="utf8")),
        #     '_p': md5_hash_passwd(password, verify),
        #     'randnumber': verify.lower(),
        #     'isPasswordPolicy': '1',
        # }
        # data = self.request(URL.get_jw_login_url(self.host), post_data)
        data = self.login_uia(username, password)
        if 200 == data['status']:
            self.login_jwc(username)
            self.login_cache(username, self.get_cookie("JSESSIONID"))
        return data

    def get_info(self):
        """
        个人信息查询
        :return:{'csrq': ‘生日’, 'xm': '姓名', 'yx': '院系', 'nj': '年级', 'zy': '计算机科学与技术14', 'xh': '学号'}
        """
        post_data = {'hidOption': 'InitData'}
        data = self.request(URL.get_jw_info_url(self.host), post_data, Rerfer.get_jw_info_refer(self.host))
        temp = lxml.html.fromstring(data.encode())
        result = dict()
        xh, nj, csrq, xm, zy, yx = "", "", "", "", "", ""
        try:
            xh = temp.xpath("xh//text()")[0]
            nj = temp.xpath("rxnj//text()")[0]
            csrq = temp.xpath("csrq//text()")[0]
            xm = temp.xpath("xm//text()")[0]
            zy = temp.xpath("bjmc//text()")[0]
            yx = temp.xpath("yxb//text()")[0]
        except Exception as e:
            print(e)
        result['xh'] = str(xh)
        result['nj'] = str(nj)
        result['csrq'] = str(csrq)
        result['xm'] = str(xm)
        result['zy'] = str(zy)
        result['yx'] = str(yx)
        return result

    def get_person_score(self, xh="", xn="", xq=""):
        """
        查询个人成绩
        :param xh:
        :param xn:
        :param xq:
        :return:{'2017年春季学期': [
        {'序号': '1', '备注': '.', '取得方式': '初修取得', '课程类别': '必修课/工作技能', '综合成绩': '优秀',
        '课程': '[080504104999]毕业设计', '学分': '12.0', '修读性质': '初修', '考核方式': '论文'}]
        """
        if xn == "" or xq == "":
            param = "xn=2017&xn1=2016&xq=1&ysyx=yscj&sjxz=sjxz1&userCode=%s&ysyxS=on&sjxzS=on" % (xh)
        else:
            param = "xn=%s&xq=%s&ysyx=yscj&sjxz=sjxz3&userCode=%s&ysyxS=on&sjxzS=on" % (xn, xq, xh)
        token = self.get_token()
        post_data = encode_param(param, token)
        data = self.request(URL.get_jw_person_score_url(self.host), post_data,
                            Rerfer.get_jw_person_score_refer(self.host))
        data = self.get_table(data)
        result = []
        for x in data:
            if len(x) == 1 and type(x[0]) == list:
                key = x[0][0].split("：")[1]
            else:
                for y in x:
                    result.append(dict(
                        name=re.sub('\[\d+\]', '', y.get("课程")),
                        xf=y.get("学分"), type=y.get("课程类别"),
                        qdfs=y.get("取得方式"), score=y.get("综合成绩"),
                        jd=cal_gpa(y.get("综合成绩")), xq=key, more=y.get("备注")))
        return result

    def get_class_score(self, xn, xq, kcdm, xkh, type=1):
        """
        查询整个班级的成绩,type为原始成绩，默认为1
        :param xn:
        :param xq:
        :param kcdm:
        :param xkh:
        :param type:
        :return:
        """
        post_data = {
            'xn': xn,
            'xq': xq,
            'kcdm': kcdm,
            'skbjdm': xkh,
            'pxfs': 'xhsx',
        }
        data = self.request(URL.get_jw_class_score_url(self.host, type), post_data, Rerfer.get_jw_info_refer(self.host))
        return self.get_table(data)

    def get_student_course(self, xh, xn, xq):
        """
        查询个人选课
        :param xh:
        :param xn:
        :param xq:
        :return:
        """
        param = "xn=%s&xq=%s&xh=%s" % (xn, xq, xh)
        param = base64.b64encode(param.encode(encoding="utf8")).decode(encoding="utf8")
        data = self.request(URL.get_jw_student_course_url(self.host) + param,
                            referer=Rerfer.get_jw_student_course_refer(self.host))
        return self.get_table(data)

    def get_school_course(self, xn, xq, kcfw="Common"):
        """
        :param xn:
        :param xq:
        :param kcfw:
        :return:
        """
        post_data = {
            'initQry': '0',
            'xn': xn,
            'xq': xq,
            'items': '',
            'kcfw': kcfw,  # kcfw: Common Specialty PublicBasic
            'sel_nj': '',
            'sel_zydm': '',
            'sel_schoolarea': '',
            'sel_cddwdm': '',
            'sel_kc': '',
            'kcmc': '',
            "tableId": "6146"
        }
        count = 1
        max_page = -1
        courses = []
        while True:
            data = self.request(URL.get_jw_scholl_course_url(self.host, count), post_data,
                                Rerfer.get_jw_school_course_refer(self.host))
            if max_page == -1:
                page = (re.findall(r'reloadPage.+\)', data))
                page = re.search(r'(\d+),(\d+)', page[0])
                max_page = int(page.group(1)) + 1
            count += 1
            data = self.get_table(data)
            if len(data) > 0:
                courses.extend(data[0])
            if count == max_page:
                break
        return courses

    def get_student_test(self, xh, xn, xq, ks_type):
        param = {
            'initQry': 0,
            'xh': xh,
            'xn': xn,
            'xq': xq,
            'xnxq': '%s-%s' % (xn, xq),
            'kslcdm': ks_type,
        }
        html = self.request(URL.get_jw_student_test_url(self.host), param, Rerfer.get_jw_student_test_refer(self.host))
        temp = self.get_table(html)
        data = []
        if len(temp):
            for x in temp[0]:
                if type(x) == list:
                    return data  # 没有考试安排 那么直接返回
                data.append(dict(
                    name=re.sub('\[\d+\]', '', x.get("课程")), type=x.get("类别"),
                    classNum=x.get("上课班号"), ksfs=x.get("统考方式"),
                    time=x.get("考试时间"), address=x.get("考试地点"),
                    num=x.get("座位号")
                ))
        return data

    def get_student_table(self, xh, xn, xq):
        params = 'xn={}&xq={}&xh={}'.format(xn, xq, xh)
        url = URL.get_jw_student_table_url(self.host) + base64.b64encode(params.encode('utf8')).decode(encoding="utf8")
        html = self.request(url, referer=Rerfer.get_jw_student_table_refer(self.host))
        return (self.get_stable(html))

    def get_course_infos(self, kcdm):
        url = URL.get_jw_course_info_url(self.host)
        params = dict(classPath="com.kingosoft.service.jw.student.pyfa.CourseInfoService", kcdm=kcdm, mtd="initCData")
        html = self.request(url, params, referer=Rerfer.get_jw_course_info_refer(self.host, kcdm))
        tmp = self.get_xml(html)["info"]["item"]
        result = None
        if tmp:
            result = dict(kcmc=tmp.get("kcmc"), kcdm=tmp.get("yhdm"), kkdw=tmp.get("cddw"))
        return result

    def get_tdkc(self, kcdm):
        url = URL.get_tdkc_url(self.host, kcdm)
        referer = Rerfer.get_jw_tdkc_refer(self.host, kcdm)
        html = self.request(url, referer=referer)
        tmp = self.get_table(html)
        return tmp


def deal_table(soup):
    temp = []
    map = []
    soup = BeautifulSoup(soup, 'html.parser')
    for tabb in soup.find_all('tr'):
        for tdd in tabb.find_all('td'):
            temp.append(tdd.get_text())
        map.append(list(temp))
        temp.clear()
    return map

==> ouc_api-2.1.1.tar.gz: ouc_api-2.1.1/ouc_api/jwCrawl.py <==
from .baseApi import BaseAPI
import random
from .urls import URL
import base64
from .utils.jkingo import md5_hash_passwd, encode_param
from .referers import Rerfer
import re
from bs4 import BeautifulSoup
from . import deyzm
import json
import redis
import lxml
import time
import requests
import os

cls = deyzm.load_classfier()


def cal_gpa(s):
    try:
        s = float(s)
        if 95 <= s <= 100:  # 95 - 100
            result = 4.0
        elif 90 <= s < 95:  # 90 - 94
            result = 3.9
        elif 85 <= s < 90:  # 85 - 89
            result = 3.7
        elif 81 <= s < 85:  # 810 - 84
            result = 3.3
        elif 78 <= s < 81:  # 78 - 79
            result = 3.0
        elif 75 <= s < 78:  # 75 -77
            result = 2.7
        elif 71 <= s < 75:  # 71- 74
            result = 2.3
        elif 68 <= s < 71:  # 68-70
            result = 2.0
        elif 64 <= s < 68:  # 64 - 67
            result = 1.7
        elif 60 <= s < 64:  # 60 - 63
            result = 1.0
        else:
            result = 0.0
    except:
        if s == "优秀":
            result = 3.9
        elif s == "良好":
            result = 3.3
        elif s == "中等":
            result = 2.3
        elif s == "合格":
            result = 1.0
        else:
            result = 0.0
    return result


class JwCrawl(BaseAPI):
    def __init__(self, xh=None, password=None, redis_host="49.234.17.183"):
        super(JwCrawl, self).__init__()
        self.redis_host = redis_host
        self.urls = [
            "http://222.195.158.208/",
        ]
        self.host = os.getenv('jwurl')
        self.sessionId = ""
        self.set_timeout(30)
        if xh and password:
            self.xh = xh
            self.login(xh, password)

    def test_host(self):
        num = 0
        while True:
            try:
                slen = len(self.urls)  # 每次都测一下长度，每次长度都会变化。
                if slen == 0:
                    self.host = ""
                    break
                num = random.randrange(0, slen)  # 随机选择一个网站
                URL = self.urls[num] + "cas/logon.action"
                self.request(URL)
                if self.get_code() == 200:
                    self.host = self.urls[num]
                    self.sessionId = self.get_cookie("JSESSIONID")  # sessionid
                    break
                else:
                    del self.urls[num]  # 该网站不可用
            except:
                del self.urls[num]  # 该网站不可用

    def get_token(self):  # 使用正则比字符串处理慢了15倍，因此采用字符串处理的方式进行。
        url = URL.get_jw_token_url(self.host)
        content = self.request(url)
        content = content.split("\n")
        _deskey = content[2].replace("var _deskey = '", "").replace("';\r", "")
        _nowtime = content[3].replace("var _nowtime = '", "").replace("';\r", "")
        # end = time.time()
        # print(end - now)
        # print(_deskey)
        # print(_nowtime)
        # d = re.findall("= (.+);", content)
        # _deskey = d[0].replace('\'', '')
        # _nowtime = d[1].replace('\'', '')
        # end = time.time()
        # print(end-now)
        data = {'_deskey': _deskey, '_nowtime': _nowtime}
        return data

    def login_cache(self, username, cookie=None):
        r = redis.Redis(self.redis_host)
        cache = r.get(username + "cookie")
        if cookie is None and cache:
            cookie = cache.decode()
            if cookie:
                self.set_cookie("JSESSIONID", cookie)
                self.sessionId = cookie
                content = self.request(self.host + "frame/desk/showSystemInfo.action")
                if username in content:
                    r.set(username + "cookie", cookie, 3600)  # 有效期绪时
                    return True
                else:
                    r.delete(username + "cookie")
                    self.http = requests.session()  # 避免产生多个jessionid的错误
                    return False
        elif username and cookie:
            r.set(username + "cookie", cookie, 3600)

    def get_code(self, xh):
        url = self.host + "cas/genValidateCode?dateTime=" + str(time.time())
        rb = self.request(url, raw=True)
        return deyzm.yzm_predict(cls, rb)

    def login(self, username="", password="", verify=""):
        self.username = username
        if self.login_cache(username):
            return dict(status="200")
        if self.sessionId is None:
            self.sessionId = ""
        if not username and password:
            return None
        verify = self.get_code(username)
        self.sessionId = self.get_cookie("JSESSIONID")
        post_data = {
            '_u' + verify: base64.b64encode((username + ";;" + self.sessionId).encode(encoding="utf8")),
            '_p' + verify: md5_hash_passwd(password, verify),
            'randnumber': verify,
            'isPasswordPolicy': '1',
        }
        data = self.request(URL.get_jw_login_url(self.host), post_data)
        data = json.loads(data)
        if data.get("status") == "200" or data.get("status") == 200:
            self.login_cache(username, self.sessionId)
        return data

    def get_info(self):
        """
        个人信息查询
        :return:{'csrq': ‘生日’, 'xm': '姓名', 'yx': '院系', 'nj': '年级', 'zy': '计算机科学与技术14', 'xh': '学号'}
        """
        post_data = {'hidOption': 'InitData'}
        data = self.request(URL.get_jw_info_url(self.host), post_data, Rerfer.get_jw_info_refer(self.host))
        temp = lxml.html.fromstring(data.encode())
        result = dict()
        xh, nj, csrq, xm, zy, yx = "", "", "", "", "", ""
        try:
            xh = temp.xpath("xh//text()")[0]
            nj = temp.xpath("rxnj//text()")[0]
            csrq = temp.xpath("csrq//text()")[0]
            xm = temp.xpath("xm//text()")[0]
            zy = temp.xpath("bjmc//text()")[0]
            yx = temp.xpath("yxb//text()")[0]
        except Exception as e:
            print(e)
        result['xh'] = str(xh)
        result['nj'] = str(nj)
        result['csrq'] = str(csrq)
        result['xm'] = str(xm)
        result['zy'] = str(zy)
        result['yx'] = str(yx)
        return result

    def get_person_score(self, xh="", xn="", xq=""):
        """
        查询个人成绩
        :param xh:
        :param xn:
        :param xq:
        :return:{'2017年春季学期': [
        {'序号': '1', '备注': '.', '取得方式': '初修取得', '课程类别': '必修课/工作技能', '综合成绩': '优秀',
        '课程': '[080504104999]毕业设计', '学分': '12.0', '修读性质': '初修', '考核方式': '论文'}]
        """
        if xn == "" or xq == "":
            param = "xn=2017&xn1=2016&xq=1&ysyx=yscj&sjxz=sjxz1&userCode=%s&ysyxS=on&sjxzS=on" % (xh)
        else:
            param = "xn=%s&xq=%s&ysyx=yscj&sjxz=sjxz3&userCode=%s&ysyxS=on&sjxzS=on" % (xn, xq, xh)
        token = self.get_token()
        post_data = encode_param(param, token)
        data = self.request(URL.get_jw_person_score_url(self.host), post_data,
                            Rerfer.get_jw_person_score_refer(self.host))
        data = self.get_table(data)
        result = []
        for x in data:
            if len(x) == 1 and type(x[0]) == list:
                key = x[0][0].split("：")[1]
            else:
                for y in x:
                    result.append(dict(
                        name=re.sub('\[\d+\]', '', y.get("课程")),
                        xf=y.get("学分"), type=y.get("课程类别"),
                        qdfs=y.get("取得方式"), score=y.get("综合成绩"),
                        jd=cal_gpa(y.get("综合成绩")), xq=key, more=y.get("备注")))
        return result

    def get_class_score(self, xn, xq, kcdm, xkh, type=1):
        """
        查询整个班级的成绩,type为原始成绩，默认为1
        :param xn:
        :param xq:
        :param kcdm:
        :param xkh:
        :param type:
        :return:
        """
        post_data = {
            'xn': xn,
            'xq': xq,
            'kcdm': kcdm,
            'skbjdm': xkh,
            'pxfs': 'xhsx',
        }
        data = self.request(URL.get_jw_class_score_url(self.host, type), post_data, Rerfer.get_jw_info_refer(self.host))
        return self.get_table(data)

    def get_student_course(self, xh, xn, xq):
        """
        查询个人选课
        :param xh:
        :param xn:
        :param xq:
        :return:
        """
        param = "xn=%s&xq=%s&xh=%s" % (xn, xq, xh)
        param = base64.b64encode(param.encode(encoding="utf8")).decode(encoding="utf8")
        data = self.request(URL.get_jw_student_course_url(self.host) + param,
                            referer=Rerfer.get_jw_student_course_refer(self.host))
        return self.get_table(data)

    def get_school_course(self, xn, xq, kcfw="Common"):
        """
        :param xn:
        :param xq:
        :param kcfw:
        :return:
        """
        post_data = {
            'initQry': '0',
            'xn': xn,
            'xq': xq,
            'items': '',
            'kcfw': kcfw,  # kcfw: Common Specialty PublicBasic
            'sel_nj': '',
            'sel_zydm': '',
            'sel_schoolarea': '',
            'sel_cddwdm': '',
            'sel_kc': '',
            'kcmc': '',
            "tableId": "6146"
        }
        count = 1
        max_page = -1
        courses = []
        while True:
            data = self.request(URL.get_jw_scholl_course_url(self.host, count), post_data,
                                Rerfer.get_jw_school_course_refer(self.host))
            if max_page == -1:
                page = (re.findall(r'reloadPage.+\)', data))
                page = re.search(r'(\d+),(\d+)', page[0])
                max_page = int(page.group(1)) + 1
            count += 1
            data = self.get_table(data)
            if len(data) > 0:
                courses.extend(data[0])
            if count == max_page:
                break
        return courses

    def get_student_test(self, xh, xn, xq, ks_type):
        param = {
            'initQry': 0,
            'xh': xh,
            'xn': xn,
            'xq': xq,
            'xnxq': '%s-%s' % (xn, xq),
            'kslcdm': ks_type,
        }
        html = self.request(URL.get_jw_student_test_url(self.host), param, Rerfer.get_jw_student_test_refer(self.host))
        temp = self.get_table(html)
        data = []
        if len(temp):
            for x in temp[0]:
                if type(x) == list:
                    return data  # 没有考试安排 那么直接返回
                data.append(dict(
                    name=re.sub('\[\d+\]', '', x.get("课程")), type=x.get("类别"),
                    classNum=x.get("上课班号"), ksfs=x.get("统考方式"),
                    time=x.get("考试时间"), address=x.get("考试地点"),
                    num=x.get("座位号")
                ))
        return data

    def get_student_table(self, xh, xn, xq):
        params = 'xn={}&xq={}&xh={}'.format(xn, xq, xh)
        url = URL.get_jw_student_table_url(self.host) + base64.b64encode(params.encode('utf8')).decode(encoding="utf8")
        html = self.request(url, referer=Rerfer.get_jw_student_table_refer(self.host))
        return (self.get_stable(html))

    def get_course_infos(self, kcdm):
        url = URL.get_jw_course_info_url(self.host)
        params = dict(classPath="com.kingosoft.service.jw.student.pyfa.CourseInfoService", kcdm=kcdm, mtd="initCData")
        html = self.request(url, params, referer=Rerfer.get_jw_course_info_refer(self.host, kcdm))
        tmp = self.get_xml(html)["info"]["item"]
        result = None
        if tmp:
            result = dict(kcmc=tmp.get("kcmc"), kcdm=tmp.get("yhdm"), kkdw=tmp.get("cddw"))
        return result

    def get_tdkc(self, kcdm):
        url = URL.get_tdkc_url(self.host, kcdm)
        referer = Rerfer.get_jw_tdkc_refer(self.host, kcdm)
        html = self.request(url, referer=referer)
        tmp = self.get_table(html)
        return tmp


def deal_table(soup):
    temp = []
    map = []
    soup = BeautifulSoup(soup, 'html.parser')
    for tabb in soup.find_all('tr'):
        for tdd in tabb.find_all('td'):
            temp.append(tdd.get_text())
        map.append(list(temp))
        temp.clear()
    return map
